#ifndef __ODEInterface_h__
#define __ODEInterface_h__

#include "Object.h"
#include "SDL/SDL.h"
#include "SDL/SDL_thread.h"
#include "WorldClock.h"

class ODEInterface {
public:
	ODEInterface(double stepsize);
	~ODEInterface();

	void RegisterObject(Object *);
	dWorldID GetWorld();
	dSpaceID GetMainSpace();

	void AquireForDeletion(Object *);

	void Pause() { pause = true; }
	void Resume() { pause = false; }
	bool Paused() { return pause; }
	void SetWarp(bool p) { warp = p; }
	void ToggleWarp() { warp = !warp; }

	void DumpWorld(string name);

private:
	
	dWorldID world;
	dSpaceID space;
	dJointGroupID contactgroup;

	list<Object *> objects;
	map<dGeomID,Object *> geom_map;

	double stepsize;
	WorldClock *wc;

	// ...
	SDL_Thread *ode_thread;
	SDL_mutex *ode_mtx;
	static int trampoline(void *t) { ((ODEInterface *)t)->PhysicsThread(); return 0; }
	void PhysicsThread();

	void lock() { SDL_mutexP(ode_mtx); }
	void unlock() { SDL_mutexV(ode_mtx); }
	
	static void spaceCollideCallback(void *t, dGeomID o1, dGeomID o2) {
		((ODEInterface *)t)->spaceCollide(o1, o2);
	}

	void spaceCollide(dGeomID o1, dGeomID o2);

	bool physics_running;
	bool pause;
	bool warp;
};

#endif
