#include <stdio.h>
#include <getopt.h>
#include "core.h"

void help(const char *argv0)
{
	fprintf(stderr, "simple_sim (called by \"%s\")\n", argv0);
	fprintf(stderr, "Options:\n");
	fprintf(stderr, "  --help, -h                    This cruft\n");
	fprintf(stderr, "  --window-size WxH, -s WxH     Window size\n");
	fprintf(stderr, "  --walls FILE, -w FILE         Walls file\n");
	fprintf(stderr, "  --targets FILE, -t FILE       Targets file\n");
	fprintf(stderr, "  --log FILE, -l FILE           Log file output\n");
	fprintf(stderr, "  --ucparams PARAMS, -p PARAMS  user_code params\n");
	fprintf(stderr, "            (PARAMS = param1=val1,param2=val2, like:\n");
	fprintf(stderr, "             \"-p mode=wp,waypointfile=wps.txt\")\n");
}

int main(int argc, char *argv[])
{
	SimCore core;
	const char *walls = "/dev/null";
	const char *targets = "/dev/null";
	const char *logfile = "/dev/null";
	const char *ucparams = "";
	int x = 640, y = 480;
	
	struct option opts [] = {
		{ "help", 0, NULL, 'h' },
		{ "walls", 1, NULL, 'w' },
		{ "window-size", 1, NULL, 's' },
		{ "targets", 1, NULL, 't' },
		{ "log", 1, NULL, 'l' },
		{ "ucparams", 1, NULL, 'p' },
		{ 0, 0, 0, 0 },
	};

	int c;
	while(1) {
		int option_index;

		c = getopt_long(argc, argv, "hw:s:t:l:p:", opts, &option_index);
		if(c == -1) break;

		switch(c) {
			case 0:
				fprintf(stderr, "huh?\n");
				break;

			case 'h':
				help(argv[0]);
				exit(0);

			case 's':
				if(sscanf(optarg, "%ix%i", &x, &y) != 2) {
					fprintf(stderr, "Option to --window-size ``%s\'\' is invalid, please use WxH\n", optarg);
					exit(-1);
				}
				break;

			case 'w':
				walls = optarg;
				break;

			case 't':
				targets = optarg;
				break;

			case 'l':
				logfile = optarg;
				break;

			case 'p':
				ucparams = optarg;
				break;
				
			case ':':
				fprintf(stderr, "missing a parameter for an option\n");
				exit(-1);
			case '?':
				exit(-1);
		}
	}

	core.LoadWallsFile(walls);
	core.LoadTargetsFile(targets);
	core.SetLoggerFile(logfile);
	core.SetWindowSize(x, y);
	core.SetUserParamsArg(ucparams);
	core.Start();

	return 0;
}
